package sun.font;

import java.security.PrivilegedAction;
import sun.java2d.DisposerRecord;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.io.IOException;
import java.security.PrivilegedExceptionAction;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.util.Iterator;
import java.lang.ref.Reference;
import sun.java2d.Disposer;
import java.io.File;
import java.nio.ByteBuffer;
import java.awt.FontFormatException;

public abstract class FileFont extends PhysicalFont
{
    protected boolean useJavaRasterizer;
    protected int fileSize;
    protected FontScaler scaler;
    protected boolean checkedNatives;
    protected boolean useNatives;
    protected NativeFont[] nativeFonts;
    protected char[] glyphToCharMap;
    
    FileFont(String s, Object o) throws FontFormatException {
        super(s, o);
        this.useJavaRasterizer = true;
    }
    
    @Override
    FontStrike createStrike(FontStrikeDesc fontStrikeDesc) {
        if (!this.checkedNatives) {
            this.checkUseNatives();
        }
        return new FileFontStrike(this, fontStrikeDesc);
    }
    
    protected boolean checkUseNatives() {
        this.checkedNatives = true;
        return this.useNatives;
    }
    
    protected abstract void close();
    
    abstract ByteBuffer readBlock(int p0, int p1);
    
    @Override
    public boolean canDoStyle(int n) {
        return true;
    }
    
    void setFileToRemove(File file, CreatedFontTracker createdFontTracker) {
        Disposer.addObjectRecord(this, new FileFont.CreatedFontFileDisposerRecord(file, createdFontTracker));
    }
    
    static void setFileToRemove(Object obj, File file, CreatedFontTracker createdFontTracker) {
        Disposer.addObjectRecord(obj, new FileFont.CreatedFontFileDisposerRecord(file, createdFontTracker));
    }
    
    synchronized void deregisterFontAndClearStrikeCache() {
        SunFontManager.getInstance().deRegisterBadFont(this);
        for (Reference reference : this.strikeCache.values()) {
            if (reference != null) {
                FileFontStrike fileFontStrike = (FileFontStrike)reference.get();
                if (fileFontStrike != null && fileFontStrike.pScalerContext != 0L) {
                    this.scaler.invalidateScalerContext(fileFontStrike.pScalerContext);
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        if (this.scaler != null) {
            this.scaler.dispose();
        }
        this.scaler = FontScaler.getNullScaler();
    }
    
    @Override
    StrikeMetrics getFontMetrics(long n) {
        try {
            return this.getScaler().getFontMetrics(n);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            return this.getFontMetrics(n);
        }
    }
    
    @Override
    float getGlyphAdvance(long n, int n2) {
        try {
            return this.getScaler().getGlyphAdvance(n, n2);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            return this.getGlyphAdvance(n, n2);
        }
    }
    
    @Override
    void getGlyphMetrics(long n, int n2, Point2D.Float float1) {
        try {
            this.getScaler().getGlyphMetrics(n, n2, float1);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            this.getGlyphMetrics(n, n2, float1);
        }
    }
    
    @Override
    long getGlyphImage(long n, int n2) {
        try {
            return this.getScaler().getGlyphImage(n, n2);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            return this.getGlyphImage(n, n2);
        }
    }
    
    @Override
    Rectangle2D.Float getGlyphOutlineBounds(long n, int n2) {
        try {
            return this.getScaler().getGlyphOutlineBounds(n, n2);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            return this.getGlyphOutlineBounds(n, n2);
        }
    }
    
    @Override
    GeneralPath getGlyphOutline(long n, int n2, float n3, float n4) {
        try {
            return this.getScaler().getGlyphOutline(n, n2, n3, n4);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            return this.getGlyphOutline(n, n2, n3, n4);
        }
    }
    
    @Override
    GeneralPath getGlyphVectorOutline(long n, int[] array, int n2, float n3, float n4) {
        try {
            return this.getScaler().getGlyphVectorOutline(n, array, n2, n3, n4);
        }
        catch (FontScalerException ex) {
            this.scaler = FontScaler.getNullScaler();
            return this.getGlyphVectorOutline(n, array, n2, n3, n4);
        }
    }
    
    protected abstract FontScaler getScaler();
    
    @Override
    protected long getUnitsPerEm() {
        return this.getScaler().getUnitsPerEm();
    }
    
    protected String getPublicFileName() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager == null) {
            return this.platName;
        }
        boolean n = true;
        try {
            securityManager.checkPropertyAccess("java.io.tmpdir");
        }
        catch (SecurityException ex) {
            n = false;
        }
        if (!(n == 0)) {
            return this.platName;
        }
        final Object o = new File(this.platName);
        Boolean false = Boolean.FALSE;
        Boolean true;
        try {
            true = (Boolean)AccessController.doPrivileged(new PrivilegedExceptionAction<Boolean>() {
                @Override
                public Boolean run() {
                    File file = new File(System.getProperty("java.io.tmpdir"));
                    try {
                        String canonicalPath = file.getCanonicalPath();
                        String canonicalPath2 = o.getCanonicalPath();
                        return Boolean.valueOf(canonicalPath2 == null || canonicalPath2.startsWith(canonicalPath));
                    }
                    catch (IOException ex) {
                        return Boolean.TRUE;
                    }
                }
            });
        }
        catch (PrivilegedActionException ex2) {
            true = Boolean.TRUE;
        }
        return ((boolean)true) ? "temp file" : this.platName;
    }
    
    private static class CreatedFontFileDisposerRecord implements DisposerRecord
    {
        File fontFile;
        CreatedFontTracker tracker;
        
        private CreatedFontFileDisposerRecord(File fontFile, CreatedFontTracker tracker) {
            this.fontFile = null;
            this.fontFile = fontFile;
            this.tracker = tracker;
        }
        
        @Override
        public void dispose() {
            AccessController.doPrivileged(new PrivilegedAction() {
                @Override
                public Object run() {
                    if (FileFont.CreatedFontFileDisposerRecord.this.fontFile != null) {
                        try {
                            if (FileFont.CreatedFontFileDisposerRecord.this.tracker != null) {
                                FileFont.CreatedFontFileDisposerRecord.this.tracker.subBytes((int)FileFont.CreatedFontFileDisposerRecord.this.fontFile.length());
                            }
                            FileFont.CreatedFontFileDisposerRecord.this.fontFile.delete();
                            SunFontManager.getInstance().tmpFontFiles.remove(FileFont.CreatedFontFileDisposerRecord.this.fontFile);
                        }
                        catch (Exception ex) {}
                    }
                    return null;
                }
            });
        }
    }
}
